@@ -58,13 +58,14 @@ mask_box_header = 0xffff
 mask_boxhdr_length = 0xffffffff00000000
 mask_boxhdr_refs_p = 0x0000000080000000
 mask_boxhdr_type = 0x000000007f000000
-mask_boxhdr_value = 0x0000000000ff0000
+mask_boxhdr_value = 0xff
 
 shift_fixnum = 3
 shift_char = 8
 shift_boolean = 8
 shift_singular = 8
 shift_void = 16
+shift_boxhdr_value = 16
 
 value_false = tag_boolean
 
@@ -149,13 +150,12 @@ class LokoGensymPrinter(LokoBoxPrinter):
         return '#<gensym>'
 
     def children(self):
+        header = deref(self.val, 0)
+        value = (header >> shift_boxhdr_value) & mask_boxhdr_value
+
         prefix = deref(self.val, 1)
         yield '0', 'prefix'
-        if prefix == value_false:
-            yield '1', prefix
-        else:
-            bv = bytes(int(ch) for _, ch in LokoBytevectorPrinter(prefix).children())
-            yield '1', bv.decode('utf-8')
+        yield '1', prefix
 
         unique = deref(self.val, 3)
         yield '2', 'unique'
@@ -165,7 +165,7 @@ class LokoGensymPrinter(LokoBoxPrinter):
             bv = bytes(int(ch) for _, ch in LokoBytevectorPrinter(unique).children())
             yield '3', bv.decode('utf-8')
 
-        yield '4', 'value'
+        yield '4', ('value' if value & 0b1 == 0b1 else 'idx')
         yield '5', deref(self.val, 4)
 
     def display_hint(self):
@@ -175,8 +175,7 @@ class LokoGensymPrinter(LokoBoxPrinter):
 class LokoSymbolPrinter(LokoBoxPrinter):
     def to_string(self):
         name = deref(self.val, 1)
-        namebv = bytes(int(ch) for _, ch in LokoBytevectorPrinter(name).children())
-        return namebv.decode('utf-8')
+        return LokoStringPrinter(name).to_string()
 
 
 class LokoPortPrinter(LokoBoxPrinter):
@@ -187,6 +186,14 @@ class LokoPortPrinter(LokoBoxPrinter):
         return 'port'
 
 
+class LokoBignumPrinter(LokoBoxPrinter):
+    def to_string(self):
+        return '#<bignum>'
+
+    def display_hint(self):
+        return 'bignum'
+
+
 class LokoRecordPrinter(LokoBoxPrinter):
     def __init__(self, val, boxheader):
         super(LokoRecordPrinter, self).__init__(val, boxheader)
@@ -220,6 +227,23 @@ class LokoRecordPrinter(LokoBoxPrinter):
         yield from recurse(self.boxheader)
 
 
+class LokoRtdPrinter(object):
+    def __init__(self, val):
+        flags_and_len = deref(val, 2) >> shift_fixnum
+        self.flags = flags_and_len >> 32
+        self.len = flags_and_len & 0xffffffff
+        self.name = deref(val, 3)
+        self.parent = deref(val, 4)
+        self.uid = deref(val, 5)
+        self.names = deref(val, 6)
+        self.mutable = deref(val, 7)
+
+    def to_string(self):
+        return '#<rtd name=%s flags=0x%x len=%d parent=%s names=%s mutable=%d>' % (
+            self.name, self.flags, self.len, self.parent,
+            self.names, self.mutable)
+
+
 class LokoProcedurePrinter(LokoPrinter):
     def __init__(self, val):
         self.val = val
@@ -308,8 +332,8 @@ def loko_lookup_function(val):
             if int(boxheader) & 0b1111 == tag_immsym:
                 # FIXME: Obsolete
                 oldtype = LokoImmsymPrinter(boxheader).to_string()
-                # if oldtype == 'symbol':
-                #     return LokoSymbolPrinter(val, boxheader)
+                if oldtype == 'rtd':
+                    return LokoRtdPrinter(val)
             if int(boxheader) & 0b111 == tag_box:
                 boxheader2 = deref(boxheader, 0)
                 oldtype2 = LokoImmsymPrinter(boxheader2).to_string()
@@ -317,12 +341,14 @@ def loko_lookup_function(val):
                     return LokoRecordPrinter(val, boxheader)
             if int(boxheader) & mask_box_header == tag_box_header:
                 if int(boxheader) & mask_boxhdr_type == btag_symbol:
-                    if int(boxheader) & mask_boxhdr_value:
+                    if (int(boxheader) >> shift_boxhdr_value) & 0b10:
                         return LokoGensymPrinter(val, boxheader)
                     else:
                         return LokoSymbolPrinter(val, boxheader)
                 elif int(boxheader) & mask_boxhdr_type == btag_port:
                     return LokoPortPrinter(val, boxheader)
+                elif int(boxheader) & mask_boxhdr_type == btag_bignum:
+                    return LokoBignumPrinter(val, boxheader)
 
             return LokoBoxPrinter(val, boxheader)
 
